      SUBROUTINE UTFDOC
C
C     UTFDOC : Dummy documentation routine for utf1314.f library.
C
C     The utf1314.f library contains argument-driven Fortran 77
C     subroutines for reading / writing UTF versions 1.3 and 1.4.
C     Each subroutine includes documentation of its arguments.
C
C     Subroutines included:
C     UTFDOC : Dummy documentation routine for utf1314.f library.
C     U13WH  : Write global header in UTF 1.3 format.
C     U13WF  : Write 2D scalar field in UTF 1.3 format.
C     U13PK  : Pack and write a data array in UTF 1.3 format.
C     U13WE  : Write end of data in UTF 1.3 format.
C     U13RH  : Read global header in UTF 1.3 format.
C     U13RF  : Read 2D scalar field in UTF 1.3 format.
C     U13UP  : Read and unpack a data array in UTF 1.3 format.
C     U14WH  : Write global header in UTF 1.4 format.
C     U14WF  : Write 2D scalar field in UTF 1.4 format.
C     U14PK  : Pack and write a data array in UTF 1.4 format.
C     U14WE  : Write end of data in UTF 1.4 format.
C     U14RH  : Read global header in UTF 1.4 format.
C     U14RF  : Read 2D scalar field in UTF 1.4 format.
C     U14UP  : Read and unpack a data array in UTF 1.3 format.
C     UTFWI  : "Write" 15 integers into an integer array for a UTF.
C              version 1.3 or 1.4 global or field header.
C     UTFRI  : "Read" 15 integers from an integer array for a UTF.
C              version 1.3 or 1.4 global or field header.
C     U13GV  : Convert a (MG,JG*NHEM) lat-long field to UTF 1.3 format
C              by repeating first longitude and adding equatorial row.
C     U13GIN : Convert a (MGP,JG*NHEM+1) lat-long field in UTF 1.3
C              format to (MG,JG*NHEM) by removing entire equatorial row
C              and copy of first longitude.
C
C     Notes:
C     1.  Only scalar 2D fields are handled - extra code must be added
C         for many of the field options and for graph fields in 1.4.
C     2.  No file handling is included - OPEN / CLOSE calls should be
C         included in the calling program.
C
C     Development history:
C     03.01.89  KD/MB           ASCOUT version for UTF 1.3.
C     28.04.93  Mike Blackburn  Argument driven UTF 1.3 routines.
C     04.05.93  Mike Blackburn  Argument driven UTF 1.4 routines.
C     24.06.93  Mike Blackburn  1.3 character pack/unpack optimised.
C     06.10.93  Mike Blackburn  New look up table in U13UP.
C     15.10.93  Mike Blackburn  U13GIN added.
C     19.06.95  Mike Blackburn  1.4 character pack/unpack optimised.
C     27.07.95  Mike Blackburn  ANSI F77 assumed size arrays.
C     05.07.96  MB/GDC/ZS       Bug-fix in U14RH.
C     18.07.96  Mike Blackburn  Combined 1.3/1.4 library, UTFWI,UTFRI.
C     05.05.98  Mike Blackburn  IASCII>2 option in U14PK, U14UP.
C                               All pack/unpack routines optimised.
C
      WRITE(*,'('' *** UTF1314 LIBRARY : DUMMY ROUTINE UTFDOC ***'')')
C
      RETURN
      END
C     ******************************************************************
      SUBROUTINE U13WH(IGH,ALON,ALAT,ALEV,CHEAD1,CHEAD2,NCHAN)
C
C     U13WH  : Write global header in UTF 1.3 format.
C
C     Input arguments:
C        IGH     -    Integer array containing 15 global switches.
C        ALON    -    Longitude array, dimension IGH(1).
C        ALAT    -    Latitude array, dimension IGH(2)*IGH(4)-IGH(6).
C        ALEV    -    Vertical level array, dimension IGH(3).
C        CHEAD1  -    First character*50 title.
C        CHEAD2  -    Second character*50 title.
C        NCHAN   -    Fortran channel for UTF output.
C     Arguments are all unchanged on output.
C
C     28.04.93  Mike Blackburn  Argument driven version.
C     27.07.95  Mike Blackburn  ANSI F77 assumed size arrays.
C
      CHARACTER*50 CHEAD1,CHEAD2,CSTART
      INTEGER      IGH(15)
      REAL         ALON(*),ALAT(*),ALEV(*)
C
      CSTART(1:50)='***START OF HEADER***    ***UTF 1.3***            '
      WRITE(NCHAN,'(A50,''.'')') CSTART(1:50)
C
      WRITE(NCHAN,'(15I5)') (IGH(I),I=1,15)
C
      INLON=IGH(1)
      INLAT=IGH(2)*IGH(4)-IGH(6)
      INLEV=IGH(3)
      WRITE(NCHAN,'(10F7.2)') (ALON(I),I=1,INLON)
      WRITE(NCHAN,'(10F7.2)') (ALAT(I),I=1,INLAT)
C
C     UTF 1.3 strictly allows only the F-format for the level array
C     but UFLUX2F/G use the E-format when INLEV>19.
C
      IF (INLEV.LE.19) THEN
         WRITE(NCHAN,'(10F7.1)') (ALEV(I),I=1,INLEV)
      ELSE
         WRITE(NCHAN,'(5(1X,1PE12.5))') (ALEV(I),I=1,INLEV)
      ENDIF
C
      WRITE(NCHAN,'(A50,''.'')') CHEAD1(1:50)
      WRITE(NCHAN,'(A50,''.'')') CHEAD2(1:50)
C
      CSTART(1:50)='***START OF DATA***      ***UTF 1.3***            '
      WRITE(NCHAN,'(A50,''.'')') CSTART(1:50)
C
      RETURN
      END
C     ******************************************************************
      SUBROUTINE U13WF(DATA,IFH,CHEAD1,CHEAD2,GMIN,GMAX,GINC,NCHAN)
C
C     U13WF  : Write 2D scalar field in UTF 1.3 format.
C
C     *** NOTE : Field type 4 (orographic mask for vertical sections)
C                is not yet coded.
C
C     Input arguments:
C        DATA    -    Real data array, dimension IFH(1)*IFH(2).
C        IFH     -    Integer array containing 15 field switches.
C        CHEAD1  -    First character*50 title.
C        CHEAD2  -    Second character*50 title.
C        GMIN    -    Minimum value of array.
C        GMAX    -    Maximum value of array.
C        GINC    -    Contour interval for plotting.
C        NCHAN   -    Fortran channel for UTF output.
C     Output arguments:
C        If GMIN, GMAX are zero on input they are computed.
C     Externals:
C        U13PK   -    Pack the 2D field.
C
C     28.04.93  Mike Blackburn  Argument driven version.
C     27.07.95  Mike Blackburn  ANSI F77 assumed size array.
C
      CHARACTER*50 CHEAD1,CHEAD2
      INTEGER      IFH(15)
      REAL         DATA(*)
C
C     Check field type is valid.
C
      IF (IFH(5).GT.4) THEN
         PRINT *,' ***WARNING IN U13WF: INVALID FIELD TYPE = ',IFH(5)
     P          ,' ENCOUNTERED : FIELD NOT WRITTEN TO UTF'
         RETURN
      ENDIF
C* TEMPORARY
      IF (IFH(5).EQ.4) THEN
         PRINT *,' ***WARNING IN U13WF: OROGRAPHIC MASK FOR VERTICAL'
     P          ,' SECTION - NOT YET CODED'
         PRINT *,' ***FIELD NOT WRITTEN TO UTF***'
         RETURN
      ENDIF
C
C     Find min and max of data if not already known.
C
      IF (GMIN.EQ.0..AND.GMAX.EQ.0.) THEN
         GMIN=DATA(1)
         GMAX=DATA(1)
         DO 10 N=2,IFH(1)*IFH(2)
            GMIN=MIN(GMIN,DATA(N))
            GMAX=MAX(GMAX,DATA(N))
   10    CONTINUE
      ENDIF
C*
C     Write header records for the field.
C
      WRITE(NCHAN,'(A50,''.'')') CHEAD1(1:50)
      WRITE(NCHAN,'(A50,''.'')') CHEAD2(1:50)
      WRITE(NCHAN,'(15I5)'     ) (IFH(I),I=1,15)
      WRITE(NCHAN,'(1P,3E15.7)') GMIN,GMAX,GINC
C
C     Pack data array and write to UTF.
C
      CALL U13PK(DATA,IFH(1)*IFH(2),GMIN,GMAX,NCHAN)
C
      RETURN
      END
C     ******************************************************************
      SUBROUTINE U13PK(DATA,NPTS,GMIN,GMAX,NCHAN)
C
C     U13PK  : Pack and write a data array in UTF 1.3 format.
C
C     Each datum is packed as two characters, with the data range
C     split into 62**2-1 bins, giving an accuracy of 1/3843 of range.
C     Uses the characters 0-9 and lower and upper case A-Z only.
C     On the first pass, a character look-up table is created and saved.
C
C     Input arguments:
C        DATA    -    Real data array.
C        NPTS    -    Dimension of array.
C        GMIN    -    Minimum value of array.
C        GMAX    -    Maximum value of array.
C        NCHAN   -    Fortran channel for UTF output.
C     Output arguments:
C        If GMIN, GMAX are zero on input they are computed.
C
C        09.87. M. A. Rowe (FRAM)             Original FRAM packing code
C     03.01.89  Kevin Dunn / Mike Blackburn.  ASCOUT version for UTF 1.3
C     24.06.93  Mike Blackburn                Argument driven version.
C     05.05.98  Mike Blackburn                Further optimisation.
C
C     Note that NPBAT must be 32, so that one record of 64 characters is
C     written in each "batch".
C
      PARAMETER(NPBAT=32,NCLEN=2*NPBAT)
      REAL       DATA(NPTS)
      INTEGER    IDATA(NCLEN)
      CHARACTER  CDATA*(NCLEN),LKUP*63
      LOGICAL    LFIRST
      SAVE LKUP,LFIRST
      DATA LFIRST/.TRUE./
C
C     ------------------------------------------------------------------
C
      IF (LFIRST) THEN
C
C        On the first call, define a look-up table of characters.
C        Table includes 0-9 and lower and upper case characters A-Z
C        but not control characters.
C        Characters 1-62 in LKUP are used to represent integer values
C        1-62. The range of the data field is scaled to 62*62 and two
C        characters used to represent each datum (2 digits in base 62).
C        63rd character in LKUP is blank for possible future masking.
C
         LKUP( 1:10)='0123456789'
         LKUP(11:36)='ABCDEFGHIJKLMNOPQRSTUVWXYZ'
         LKUP(37:62)='abcdefghijklmnopqrstuvwxyz'
         LKUP(63:63)=' '
         LFIRST=.FALSE.
C
      ENDIF
C
C     ------------------------------------------------------------------
C
C     Find min and max of data if not already known.
C
      IF (GMIN.EQ.0..AND.GMAX.EQ.0.) THEN
         GMIN=DATA(1)
         GMAX=DATA(1)
         DO 10 N=2,NPTS
            GMIN=MIN(GMIN,DATA(N))
            GMAX=MAX(GMAX,DATA(N))
   10    CONTINUE
      ENDIF
C
C     Find scaling factor.  Set to zero for zero field range.
C
      GAMAX=MAX(ABS(GMAX),ABS(GMIN))
      RANGE=GMAX-GMIN
      ARANG=62.*62.-1.
      IF (RANGE.LE.GAMAX*1.E-10) THEN
         SCALE=0.
      ELSE
         SCALE=ARANG/RANGE
      ENDIF
C
C     Split packing and writing into batches of NPBAT points,
C     since packed data array size is defined by this parameter.
C
      NBAT=NPTS/NPBAT
      NLEFT=MOD(NPTS,NPBAT)
      IF (NLEFT.EQ.0) THEN
         NBAT=NBAT-1
         NLEFT=NPBAT
      ENDIF
C
C     Scale data using range of field and encode each datum
C     using subscript of look-up table character.
C
      IPTS=0
      NP=NPBAT
      DO 40 IBAT=1,NBAT+1
         IF (IBAT.EQ.NBAT+1) NP=NLEFT
         DO 20 N1=1,NP*2,2
            IPTS=IPTS+1
            I=NINT((DATA(IPTS)-GMIN)*SCALE)
            IDATA(N1  )=1+I/62
            IDATA(N1+1)=1+MOD(I,62)
   20    CONTINUE
         DO 30 N=1,NP*2
            I=IDATA(N)
            CDATA(N:N)=LKUP(I:I)
   30    CONTINUE
         WRITE(NCHAN,'(A)') CDATA(1:2*NP)
   40 CONTINUE
C
      RETURN
      END
C     ******************************************************************
      SUBROUTINE U13WE(NCHAN)
C
C     U13WE  : Write end of data in UTF 1.3 format.
C
C     Input arguments:
C        NCHAN   -    Fortran channel for UTF output.
C     (Unchanged on output).
C
C     28.04.93  Mike Blackburn  Argument driven version.
C
      CHARACTER*50 CEND
C
      CEND(1:50)='***END OF DATA***        ***UTF 1.3***            '
      WRITE(NCHAN,'(A50,''.'')') CEND(1:50)
C
      RETURN
      END
C     ******************************************************************
      SUBROUTINE U13RH(IGH,ALON,ALAT,ALEV,CHEAD1,CHEAD2,NCHAN,IERR)
C
C     U13RH  : Read global header in UTF 1.3 format.
C
C     Input arguments:
C        ALON    -    Must be dimensioned for longitudes of input UTF.
C        ALAT    -    Must be dimensioned for latitudes of input UTF.
C        ALEV    -    Must be dimensioned for levels of input UTF.
C        NCHAN   -    Fortran channel for UTF input.
C        Remaining arguments are all unused and overwritten on output.
C    Output arguments:
C        IGH     -    Integer array containing 15 global switches.
C        ALON    -    Longitude array, dimension IGH(1).
C        ALAT    -    Latitude array, dimension IGH(2)*IGH(4)-IGH(6).
C        ALEV    -    Vertical level array, dimension IGH(3).
C        CHEAD1  -    First character*50 title.
C        CHEAD2  -    Second character*50 title.
C        NCHAN   -    (unchanged).
C        IERR }  = 0  Successful completion
C             }  = 1  End of file encountered.
C             }  = 2  Incorrect UTF version encountered.
C
C     28.04.93  Mike Blackburn  Argument driven version.
C
      CHARACTER*50 CHEAD1,CHEAD2,CSTART,CBLANK,CREAD
      INTEGER      IGH(15)
      REAL         ALON(*),ALAT(*),ALEV(*)
C
C     Check for start of global header.
C
      CSTART(1:50)='***START OF HEADER***    ***UTF 1.3***            '
      CBLANK(1:50)='                                                  '
   10 CONTINUE
      READ (NCHAN,'(A50)',END=90) CREAD(1:50)
      IF (CREAD(1:21).NE.CSTART(1:21)) GOTO 10
      IF (CREAD(26:38).NE.CSTART(26:38).AND.
     I    CREAD(26:38).NE.CBLANK(26:38)) THEN
         PRINT *,' ***U13RH IERR=2 : UTF VERSION IS NOT 1.3.'
     P          ,'  START-OF-GLOBAL-HEADER RECORD IS:'
         PRINT *,CREAD(1:50)
         IERR=2
         RETURN
      ENDIF
C
C     Read data records from global header.
C
      READ (NCHAN,'(15I5)',END=90) (IGH(I),I=1,15)
      INLON=IGH(1)
      INLAT=IGH(2)*IGH(4)-IGH(6)
      INLEV=IGH(3)
C
      READ (NCHAN,'(10F7.2)',END=90) (ALON(I),I=1,INLON)
      READ (NCHAN,'(10F7.2)',END=90) (ALAT(I),I=1,INLAT)
C
C     UTF 1.3 strictly allows only the F-format for the level array
C     but UFLUX2F/G use the E-format when INLEV>19.
C
      IF (INLEV.LE.19) THEN
         READ (NCHAN,'(10F7.1)',END=90) (ALEV(I),I=1,INLEV)
      ELSE
         READ (NCHAN,'(5(1X,1PE12.5))',END=90) (ALEV(I),I=1,INLEV)
      ENDIF
C
      READ (NCHAN,'(A50)',END=90) CHEAD1(1:50)
      READ (NCHAN,'(A50)',END=90) CHEAD2(1:50)
C
C     Check for start of data record and position after it.
C
      CSTART(1:50)='***START OF DATA***      ***UTF 1.3***            '
   20 CONTINUE
      READ (NCHAN,'(A50)',END=90) CREAD(1:50)
      IF (CREAD(1:19).NE.CSTART(1:19)) GOTO 20
C
      IERR=0
      RETURN
C
C     End of file encountered.
C
   90 CONTINUE
      PRINT *,' ***U13RH IERR=1 : END OF FILE ENCOUNTERED READING'
     P       ,' GLOBAL HEADER ON CHANNEL ',NCHAN
      PRINT *,' SUSPECT INCORRECT FORMAT'
      IERR=1
      RETURN
C
      END
C     ******************************************************************
      SUBROUTINE U13RF(DATA,IFH,CHEAD1,CHEAD2,GMIN,GMAX,GINC,NCHAN,IERR)
C
C     U13RF  : Read 2D scalar field in UTF 1.3 format.
C
C     Check for end of data.
C     *** NOTE : Field type 4 (orographic mask for vertical sections)
C                is not yet coded.
C
C     Input arguments:
C        DATA    -    Must be dimensioned to receive 2D field.
C        NCHAN   -    Fortran channel for UTF input.
C        Remaining arguments are all unused and overwritten on output.
C     Output arguments:
C        DATA    -    Real data array, dimension IFH(1)*IFH(2).
C        IFH     -    Integer array containing 15 field switches.
C        CHEAD1  -    First character*50 title.
C        CHEAD2  -    Second character*50 title.
C        GMIN    -    Minimum value of array.
C        GMAX    -    Maximum value of array.
C        GINC    -    Contour interval for plotting.
C        NCHAN   -    (unchanged).
C        IERR }  = 0  Successful completion
C             }  =-1  Field has zero range (soft error).
C             }  = 1  End of file encountered.
C             }  = 2  Invalid character encountered in data.
C             }  = 3  End of UTF found.
C             }  = 4  Invalid field type.
C     Externals:
C        U13UP   -    Unpack the 2D field.
C
C     28.04.93  Mike Blackburn  Argument driven version.
C
      CHARACTER*50 CHEAD1,CHEAD2,CBLANK,CEND
      INTEGER      IFH(15)
      REAL         DATA(*)
C
      CBLANK(1:50)='                                                  '
      CEND  (1:50)='***END OF DATA***        ***UTF 1.3***            '
      CHEAD1=CBLANK
      CHEAD2=CBLANK
C
C     Read header records.
C
      READ (NCHAN,'(A50)',END=90) CHEAD1(1:50)
      IF (CHEAD1(1:17).EQ.CEND(1:17)) THEN
         IERR=3
         RETURN
      ENDIF
      READ (NCHAN,'(A50)',END=90) CHEAD2(1:50)
      READ (NCHAN,'(15I5)'     ,END=90) (IFH(I),I=1,15)
      READ (NCHAN,'(1P,3E15.7)',END=90) GMIN,GMAX,GINC
C
C     Check field type is valid.
C
      IF (IFH(5).GT.4) THEN
         IERR=4
         RETURN
      ENDIF
C* TEMPORARY
      IF (IFH(5).EQ.4) THEN
         PRINT *,' ***WARNING IN U13RF: OROGRAPHIC MASK FOR VERTICAL'
     P          ,' SECTION - NOT YET CODED'
         IERR=4
         RETURN
      ENDIF
C
C     Read and unpack data array.
C     IERR is set by U13UP.
C
      CALL U13UP(DATA,IFH(1)*IFH(2),GMIN,GMAX,NCHAN,IERR)
      IF (IERR.NE.0) GOTO 99
C
      RETURN
C
C     End of file encountered.
C
   90 CONTINUE
      PRINT *,' ***U13RF IERR=1 : END OF FILE ENCOUNTERED READING'
     P       ,' FIELD ON CHANNEL ',NCHAN
      IERR=1
C
C     Print field header unless blank if there is an error.
C
   99 CONTINUE
      IF (CHEAD1.NE.CBLANK) THEN
         PRINT *,'FIELD IS :'
         PRINT *,CHEAD1(1:50)
         PRINT *,CHEAD2(1:50)
      ENDIF
      RETURN
C
      END
C     ******************************************************************
      SUBROUTINE U13UP(DATA,NPTS,GMIN,GMAX,NCHAN,IERR)
C
C     U13UP  : Read and unpack a data array in UTF 1.3 format.
C
C     Each datum is packed as two characters, with the data range
C     split into 62**2-1 bins, giving an accuracy of 1/3843 of range.
C     Uses the characters 0-9 and lower and upper case A-Z only.
C     On the first pass, a look-up table is created and saved.
C
C     Input arguments:
C        DATA    -    Must be dimensioned NPTS to receive data.
C        NPTS    -    Number of data values.
C        GMIN    -    Minimum value of array.
C        GMAX    -    Maximum value of array.
C        NCHAN   -    Fortran channel for UTF input.
C     Output arguments:
C        DATA    -    Real data array.
C        IERR }  = 0  Successful completion
C             }  =-1  Field has zero range (soft error).
C             }  = 1  End of file encountered.
C             }  = 2  Invalid character encountered in data.
C        Other arguments unchanged.
C
C        09.87. M. A. Rowe (FRAM)             Original FRAM packing code
C     03.01.89  Kevin Dunn / Mike Blackburn.  ASCOUT version for UTF 1.3
C     24.06.93  Mike Blackburn                Argument driven version.
C     06.10.93  Mike Blackburn                New look up table based on
C                                             Conrad Winchester's code.
C     05.05.98  Mike Blackburn                Further optimisation.
C
C     Note that NPBAT must be 32, so that one record of 64 characters is
C     read in each "batch".
C
      PARAMETER (NPBAT=32,NCLEN=2*NPBAT)
      REAL       DATA(NPTS)
      INTEGER    ITAB1(256),IDATA(NCLEN)
      CHARACTER  CDATA*(NCLEN),LKUP*63
      LOGICAL    LFIRST
      SAVE ITAB1,LFIRST
      DATA LFIRST/.TRUE./
C
C     ------------------------------------------------------------------
C
      IF (LFIRST) THEN
C
C        On the first call, define a look-up table of characters.
C        Table includes 0-9 and lower and upper case characters A-Z
C        but not control characters.
C        Characters 1-62 in LKUP are used to represent integer values
C        1-62. The range of the data field is scaled to 62*62 and two
C        characters used to represent each datum (2 digits in base 62).
C        63rd character in LKUP is blank for possible future masking.
C
         LKUP( 1:10)='0123456789'
         LKUP(11:36)='ABCDEFGHIJKLMNOPQRSTUVWXYZ'
         LKUP(37:62)='abcdefghijklmnopqrstuvwxyz'
         LKUP(63:63)=' '
C
C        Convert characters to an integer look-up table.
C        The integer value of each character in the collating sequence
C        is used as the index of the table.  Unused characters have a
C        value of 999.
C
         DO 10 I=1,256
            ITAB1(I)=999
   10    CONTINUE
         DO 20 I=1,62
            ICOLL=ICHAR(LKUP(I:I))
            ITAB1(ICOLL)=I
   20    CONTINUE
C
         LFIRST=.FALSE.
C
      ENDIF
C
C     ------------------------------------------------------------------
C
C     Obtain range and scaling factor.
C
      GAMAX=MAX(ABS(GMAX),ABS(GMIN))
      RANGE=GMAX-GMIN
      ARANG=62.*62.-1.
      SCALE=RANGE/ARANG
C
C     Split reading and unpacking into batches of NPBAT points,
C     since packed data array size is defined by this parameter.
C
      NBAT=NPTS/NPBAT
      NLEFT=MOD(NPTS,NPBAT)
      IF (NLEFT.EQ.0) THEN
         NBAT=NBAT-1
         NLEFT=NPBAT
      ENDIF
C
C     Decode data using subscript of character look-up table
C     to obtain scaled integers.
C     Rescale using range of field to obtain floating point value.
C
      IPTS=0
      NP=NPBAT
      DO 50 IBAT=1,NBAT+1
         IF (IBAT.EQ.NBAT+1) NP=NLEFT
         READ (NCHAN,'(A)',END=90) CDATA(1:2*NP)
         ICMAX=0
         DO 30 N=1,NP*2
            IDATA(N)=ITAB1(ICHAR(CDATA(N:N)))
   30    CONTINUE
         DO 40 N1=1,NP*2,2
            IPTS=IPTS+1
            I1=IDATA(N1  )
            I0=IDATA(N1+1)
            I=(I1-1)*62+(I0-1)
            DATA(IPTS)=REAL(I)*SCALE+GMIN
            ICMAX=MAX(ICMAX,I1,I0)
   40    CONTINUE
         IF (ICMAX.EQ.999) GOTO 91
   50 CONTINUE
C
C     Print warning if range of field is zero.
C     This is treated as a soft error (IERR<0).
C     Otherwise return successfully.
C
      IERR=0
      IF (RANGE.LE.GAMAX*1.E-10) THEN
         PRINT *,' ***U13UP WARNING : FIELD HAS ZERO RANGE.'
         IERR=-1
      ENDIF
      RETURN
C
C     End of file encountered.
C
   90 CONTINUE
      PRINT *,' ***U13UP IERR=1 : END OF FILE ENCOUNTERED READING'
     P       ,' FIELD ON CHANNEL ',NCHAN
      IERR=1
      RETURN
C
C     Invalid character encountered in data.
C
   91 CONTINUE
      PRINT *,' ***U13UP IERR=2 : INVALID CHARACTER ENCOUNTERED READING'
     P       ,' DATA ON CHANNEL ',NCHAN
      PRINT *,' RECORD IS :'
      WRITE(*,'(A)') CDATA(1:2*NP)
      IERR=2
      RETURN
C
      END
C     ******************************************************************
      SUBROUTINE U14WH(IGH,ALON,ALAT,ALEV,ATIME,CHEAD1,CHEAD2,NCHAN)
C
C     U14WH  : Write global header in UTF 1.4 format.
C
C     Input arguments:
C        IGH     -    Integer array containing 15 global switches.
C        ALON    -    Longitude array, dimension IGH(1).
C        ALAT    -    Latitude array, dimension IGH(2)*IGH(4)-IGH(6).
C        ALEV    -    Vertical level array, dimension IGH(3).
C        ATIME   -    Time array, dimension IGH(11), missing if zero.
C        CHEAD1  -    First character*50 title.
C        CHEAD2  -    Second character*50 title.
C        NCHAN   -    Fortran channel for UTF output.
C     Arguments are all unchanged on output.
C
C     04.05.93  Mike Blackburn  Argument driven version.
C     17.07.95  Mike Blackburn  ANSI F77 assumed size arrays.
C
      CHARACTER*50 CHEAD1,CHEAD2,CSTART
      INTEGER      IGH(15)
      REAL         ALON(*),ALAT(*),ALEV(*),ATIME(*)
C
      CSTART(1:50)='***START OF HEADER***    ***UTF 1.4***            '
      WRITE(NCHAN,'(A50,''$'')') CSTART(1:50)
C
      WRITE(NCHAN,'(15I5)') (IGH(I),I=1,15)
C
      INLON=IGH(1)
      INLAT=IGH(2)*IGH(4)-IGH(6)
      INLEV=IGH(3)
      WRITE(NCHAN,'(10F7.2)') (ALON(I),I=1,INLON)
      WRITE(NCHAN,'(10F7.2)') (ALAT(I),I=1,INLAT)
C
      WRITE(NCHAN,'(1P,5(1X,E12.5))') (ALEV(I),I=1,INLEV)
C
      IF (IGH(11).GT.0) THEN
         WRITE(NCHAN,'(5(1X,F13.5))') (ATIME(I),I=1,IGH(11))
      ENDIF
C
      WRITE(NCHAN,'(A50,''$'')') CHEAD1(1:50)
      WRITE(NCHAN,'(A50,''$'')') CHEAD2(1:50)
C
      CSTART(1:50)='***START OF DATA***      ***UTF 1.4***            '
      WRITE(NCHAN,'(A50,''$'')') CSTART(1:50)
C
      RETURN
      END
C     ******************************************************************
      SUBROUTINE U14WF(DATA,IFH,CHEAD1,CHEAD2,GMIN,GMAX,GINC,NCHAN)
C
C     U14WF  : Write 2D scalar field in UTF 1.4 format.
C
C     *** NOTE : Field type 4 (orographic mask for vertical sections)
C                is not yet coded.
C     *** NOTE : IPTYP=0 not yet coded (would need axes).
C
C     Input arguments:
C        DATA    -    Real data array, dimension IFH(1)*IFH(2).
C        IFH     -    Integer array containing 15 field switches.
C        CHEAD1  -    First character*50 title.
C        CHEAD2  -    Second character*50 title.
C        GMIN    -    Minimum value of array.
C        GMAX    -    Maximum value of array.
C        GINC    -    Contour interval for plotting.
C        NCHAN   -    Fortran channel for UTF output.
C     Output arguments:
C        If GMIN, GMAX are zero on input they are computed.
C     Externals:
C        U14PK   -    Pack the 2D field.
C
C     04.05.93  Mike Blackburn  Argument driven version.
C     17.07.95  Mike Blackburn  ANSI F77 assumed size array.
C     05.05.98  Mike Blackburn  Remove IASCII=2 restriction.
C
      IMPLICIT LOGICAL (L)
C
      CHARACTER*50 CHEAD1,CHEAD2
      INTEGER      IFH(15)
      REAL         DATA(*)
C
C     Check plot and field types are valid.
C
      LVALID=.TRUE.
      IF (IFH(3).LT.0) LVALID=.FALSE.
      IF (IFH(3).GT.24) LVALID=.FALSE.
      IF (IFH(3).GE.13.AND.IFH(3).LE.20) LVALID=.FALSE.
      IF (.NOT.LVALID) THEN
         PRINT *,' ***WARNING IN U14WF: INVALID PLOT TYPE = ',IFH(3)
     P          ,' ENCOUNTERED : FIELD NOT WRITTEN TO UTF'
         RETURN
      ENDIF
C* TEMPORARY
      IF (IFH(3).EQ.0) THEN
         PRINT *,' ***WARNING IN U14WF: PLOT TYPE ZERO REQUIRES AXES'
     P          ,' - NOT YET CODED'
         PRINT *,' ***FIELD NOT WRITTEN TO UTF***'
         RETURN
      ENDIF
C
      IF (IFH(5).GT.4) THEN
         PRINT *,' ***WARNING IN U14WF: INVALID FIELD TYPE = ',IFH(5)
     P          ,' ENCOUNTERED : FIELD NOT WRITTEN TO UTF'
         RETURN
      ENDIF
C* TEMPORARY
      IF (IFH(5).EQ.4) THEN
         PRINT *,' ***WARNING IN U14WF: OROGRAPHIC MASK FOR VERTICAL'
     P          ,' SECTION - NOT YET CODED'
         PRINT *,' ***FIELD NOT WRITTEN TO UTF***'
       RETURN
      ENDIF
C
C     Find min and max of data if not already known.
C
      IF (GMIN.EQ.0..AND.GMAX.EQ.0.) THEN
         GMIN=DATA(1)
         GMAX=DATA(1)
         DO 10 N=2,IFH(1)*IFH(2)
            GMIN=MIN(GMIN,DATA(N))
            GMAX=MAX(GMAX,DATA(N))
   10    CONTINUE
      ENDIF
C*
C     Write header records for the field.
C
      WRITE(NCHAN,'(A50,''$'')') CHEAD1(1:50)
      WRITE(NCHAN,'(A50,''$'')') CHEAD2(1:50)
      WRITE(NCHAN,'(15I5)'     ) (IFH(I),I=1,15)
      WRITE(NCHAN,'(1P,3E15.7)') GMIN,GMAX,GINC
C
C     Pack data array and write to UTF.
C
      CALL U14PK(DATA,IFH(1)*IFH(2),IFH(10),GMIN,GMAX,NCHAN)
C
      RETURN
      END
C     ******************************************************************
      SUBROUTINE U14PK(DATA,NPTS,IASCII,GMIN,GMAX,NCHAN)
C
C     U14PK  : Pack and write a data array in UTF 1.4 format.
C
C     Each datum is packed as IASCII characters, with the data range
C     split into K**IASCII-1 bins, giving an accuracy of approximately
C     1/(K**IASCII) of the range.
C     Uses the characters 0-9 and lower and upper case A-Z only (K=62).
C     On the first pass, a character look-up table is created and saved.
C     *** NOTE : only coded for IASCII=2, 3, or 4.
C     *** NOTE : missing data not yet coded.
C
C     Input arguments:
C        DATA    -    Real data array.
C        NPTS    -    Dimension of array.
C        IASCII  -    Number of characters per datum.
C        GMIN    -    Minimum value of array.
C        GMAX    -    Maximum value of array.
C        NCHAN   -    Fortran channel for UTF output.
C     Output arguments:
C        If GMIN, GMAX are zero on input they are computed.
C
C        09.87. M. A. Rowe (FRAM)             Original FRAM packing code
C     03.01.89  Kevin Dunn / Mike Blackburn.  ASCOUT version for UTF 1.3
C     04.05.93  Mike Blackburn                Argument driven version.
C     19.06.95  Mike Blackburn                Character array from UTF13
C                                             code.
C     05.05.98  Mike Blackburn                Coded for IASCII=2,3,or 4.
C                                             Write in 3-record batches.
C
      PARAMETER (K=62,K2=K*K,K3=K2*K,KLEN=K+1
     *          ,NRLEN=64,NCLEN=3*NRLEN)
      REAL       DATA(NPTS)
      INTEGER    IDATA(NCLEN)
      CHARACTER  CDATA*(NCLEN),LKUP*(KLEN)
      LOGICAL    LFIRST
      SAVE LKUP,LFIRST
      DATA LFIRST/.TRUE./
C
C     ------------------------------------------------------------------
C
      IF (LFIRST) THEN
C
C        On the first call, define a look-up table of characters.
C        Table includes 0-9 and lower and upper case characters A-Z
C        but not control characters (K=62).
C        Characters 1 to K in LKUP are used to represent integer values
C        0 to K-1. The range of the data field is scaled to K**IASCII
C        and IASCII characters used to represent each datum as IASCII
C        digits in base K.
C        (K+1)th character in LKUP is blank for possible future masking.
C
         LKUP( 1:10)='0123456789'
         LKUP(11:36)='ABCDEFGHIJKLMNOPQRSTUVWXYZ'
         LKUP(37:62)='abcdefghijklmnopqrstuvwxyz'
         LKUP(KLEN:KLEN)=' '
         LFIRST=.FALSE.
C
      ENDIF
C
C     ------------------------------------------------------------------
C
C     Check number of characters per datum.
C
      IF (IASCII.LT.2) THEN
         PRINT *,' ***ABORT IN U14PK: INVALID IASCII < 2'
         CALL ABORT
      ELSEIF (IASCII.GT.4) THEN
         PRINT *,' ***ABORT IN U14PK: IASCII > 4 NOT CODED'
         CALL ABORT
      ENDIF
C
C     Find min and max of data if not already known.
C
      IF (GMIN.EQ.0..AND.GMAX.EQ.0.) THEN
         GMIN=DATA(1)
         GMAX=DATA(1)
         DO 10 N=2,NPTS
            GMIN=MIN(GMIN,DATA(N))
            GMAX=MAX(GMAX,DATA(N))
   10    CONTINUE
      ENDIF
C
C     Find scaling factor.  Set to zero for zero field range.
C
      GAMAX=MAX(ABS(GMAX),ABS(GMIN))
      RANGE=GMAX-GMIN
      ARANG=REAL(K**IASCII-1)
      IF (RANGE.LE.GAMAX*1.E-10) THEN
         SCALE=0.
      ELSE
         SCALE=ARANG/RANGE
      ENDIF
C
C     Split packing and writing into batches of NPBAT points,
C     since packed data array size is defined by this parameter.
C
      NPBAT=NCLEN/IASCII
      NBAT=NPTS/NPBAT
      NLEFT=MOD(NPTS,NPBAT)
      IF (NLEFT.EQ.0) THEN
         NBAT=NBAT-1
         NLEFT=NPBAT
      ELSE
C        Calculate the number of characters and records in any final
C        partial batch.
         NCHAR=NLEFT*IASCII
         NREC=NCHAR/NRLEN
         NREM=MOD(NCHAR,NRLEN)
      ENDIF
C
C     Scale data using range of field and encode each datum
C     using subscript of look-up table character.
C
      IPTS=0
      NP=NPBAT
      DO 60 IBAT=1,NBAT+1
         IF (IBAT.EQ.NBAT+1) NP=NLEFT
C
         IF (IASCII.EQ.2) THEN
            DO 20 N1=1,NP*2,2
               IPTS=IPTS+1
               I=NINT((DATA(IPTS)-GMIN)*SCALE)
               IDATA(N1  )=1+I/K
               IDATA(N1+1)=1+MOD(I,K)
   20       CONTINUE
         ELSEIF (IASCII.EQ.3) THEN
            DO 30 N1=1,NP*3,3
               IPTS=IPTS+1
               I=NINT((DATA(IPTS)-GMIN)*SCALE)
               IDATA(N1  )=1+I/K2
               IDATA(N1+1)=1+(MOD(I,K2)/K)
               IDATA(N1+2)=1+MOD(I,K)
   30       CONTINUE
         ELSEIF (IASCII.EQ.4) THEN
            DO 40 N1=1,NP*4,4
               IPTS=IPTS+1
               I=NINT((DATA(IPTS)-GMIN)*SCALE)
               IDATA(N1  )=1+I/K3
               IDATA(N1+1)=1+(MOD(I,K3)/K2)
               IDATA(N1+2)=1+(MOD(I,K2)/K)
               IDATA(N1+3)=1+MOD(I,K)
   40       CONTINUE
         ENDIF
C
         DO 45 N=1,NP*IASCII
            I=IDATA(N)
            CDATA(N:N)=LKUP(I:I)
   45    CONTINUE
C
         IF (NP.EQ.NPBAT) THEN
            WRITE(NCHAN,'(A/A/A)') CDATA(1:NRLEN),CDATA(NRLEN+1:2*NRLEN)
     *                            ,CDATA(2*NRLEN+1:3*NRLEN)
         ELSE
            IF (NREC.GT.0) THEN
               DO 50 IREC=1,NREC
                  WRITE(NCHAN,'(A)') CDATA((IREC-1)*NRLEN+1:IREC*NRLEN)
   50          CONTINUE
            ENDIF
            IF (NREM.GT.0) THEN
               WRITE(NCHAN,'(A)') CDATA(NREC*NRLEN+1:NREC*NRLEN+NREM)
            ENDIF
         ENDIF
C
   60 CONTINUE
C
C     Check that correct number of data points have been processed.
C
      IF (IPTS.NE.NPTS) THEN
         PRINT *,'***ABORT IN U14PK: WRONG COUNTER SUM: IPTS,NPTS='
     *          ,IPTS,NPTS
         CALL ABORT
      ENDIF
C
      RETURN
      END
C     ******************************************************************
      SUBROUTINE U14WE(NCHAN)
C
C     U14WE  : Write end of data in UTF 1.4 format.
C
C     Input arguments:
C        NCHAN   -    Fortran channel for UTF output.
C     (Unchanged on output).
C
C     04.05.93  Mike Blackburn  Argument driven version.
C
      CHARACTER*50 CEND
C
      CEND(1:50)='***END OF DATA***        ***UTF 1.4***            '
      WRITE(NCHAN,'(A50,''$'')') CEND(1:50)
C
      RETURN
      END
C     ******************************************************************
      SUBROUTINE U14RH(IGH,ALON,ALAT,ALEV,ATIME,CHEAD1,CHEAD2,NCHAN
     S                ,IERR)
C
C     U14RH  : Read global header in UTF 1.4 format.
C
C     Input arguments:
C        ALON    -    Must be dimensioned for longitudes of input UTF.
C        ALAT    -    Must be dimensioned for latitudes of input UTF.
C        ALEV    -    Must be dimensioned for levels of input UTF.
C        ATIME   -    Must be dimensioned for times of input UTF.
C        NCHAN   -    Fortran channel for UTF input.
C        Remaining arguments are all unused and overwritten on output.
C    Output arguments:
C        IGH     -    Integer array containing 15 global switches.
C        ALON    -    Longitude array, dimension IGH(1).
C        ALAT    -    Latitude array, dimension IGH(2)*IGH(4)-IGH(6).
C        ALEV    -    Vertical level array, dimension IGH(3).
C        ATIME   -    Time array, dimension IGH(11)
C        CHEAD1  -    First character*50 title.
C        CHEAD2  -    Second character*50 title.
C        NCHAN   -    (unchanged).
C        IERR }  = 0  Successful completion
C             }  = 1  End of file encountered.
C             }  = 2  Incorrect UTF version encountered.
C
C     04.05.93  Mike Blackburn Argument driven version.
C     05.07.96  MB/GDC/ZS      Read time array only if IGH(11) non-zero.
C
      CHARACTER*50 CHEAD1,CHEAD2,CSTART,CBLANK,CREAD
      INTEGER      IGH(15)
      REAL         ALON(*),ALAT(*),ALEV(*),ATIME(*)
C
C     Check for start of global header.
C
      CSTART(1:50)='***START OF HEADER***    ***UTF 1.4***            '
      CBLANK(1:50)='                                                  '
   10 CONTINUE
      READ (NCHAN,'(A50)',END=90) CREAD(1:50)
      IF (CREAD(1:21).NE.CSTART(1:21)) GOTO 10
      IF (CREAD(26:38).NE.CSTART(26:38).AND.
     I    CREAD(26:38).NE.CBLANK(26:38)) THEN
         PRINT *,' ***U14RH IERR=2 : UTF VERSION IS NOT 1.4.'
     P          ,'  START-OF-GLOBAL-HEADER RECORD IS:'
         PRINT *,CREAD(1:50)
         IERR=2
         RETURN
      ENDIF
C
C     Read data records from global header.
C
      READ (NCHAN,'(15I5)',END=90) (IGH(I),I=1,15)
      INLON=IGH(1)
      INLAT=IGH(2)*IGH(4)-IGH(6)
      INLEV=IGH(3)
C
      READ (NCHAN,'(10F7.2)',END=90) (ALON(I),I=1,INLON)
      READ (NCHAN,'(10F7.2)',END=90) (ALAT(I),I=1,INLAT)
C
      READ (NCHAN,'(1P,5(1X,E12.5))',END=90) (ALEV(I),I=1,INLEV)
C
      IF (IGH(11).NE.0) THEN
         READ (NCHAN,'(5(1X,F13.5))',END=90) (ATIME(I),I=1,IGH(11))
      ENDIF
C
      READ (NCHAN,'(A50)',END=90) CHEAD1(1:50)
      READ (NCHAN,'(A50)',END=90) CHEAD2(1:50)
C
C     Check for start of data record and position after it.
C
      CSTART(1:50)='***START OF DATA***      ***UTF 1.4***            '
   20 CONTINUE
      READ (NCHAN,'(A50)',END=90) CREAD(1:50)
      IF (CREAD(1:19).NE.CSTART(1:19)) GOTO 20
C
      IERR=0
      RETURN
C
C     End of file encountered.
C
   90 CONTINUE
      PRINT *,' ***U14RH IERR=1 : END OF FILE ENCOUNTERED READING'
     P       ,' GLOBAL HEADER ON CHANNEL ',NCHAN
      PRINT *,' SUSPECT INCORRECT FORMAT'
      IERR=1
      RETURN
C
      END
C     ******************************************************************
      SUBROUTINE U14RF(DATA,IFH,CHEAD1,CHEAD2,GMIN,GMAX,GINC,NCHAN,IERR)
C
C     U14RF  : Read 2D scalar field in UTF 1.4 format.
C
C     Check for end of data.
C     *** NOTE : Field type 4 (orographic mask for vertical sections)
C                is not yet coded.
C     *** NOTE : IPTYP=0 not yet coded (would need axes).
C
C     Input arguments:
C        DATA    -    Must be dimensioned to receive 2D field.
C        NCHAN   -    Fortran channel for UTF input.
C        Remaining arguments are all unused and overwritten on output.
C     Output arguments:
C        DATA    -    Real data array, dimension IFH(1)*IFH(2).
C        IFH     -    Integer array containing 15 field switches.
C        CHEAD1  -    First character*50 title.
C        CHEAD2  -    Second character*50 title.
C        GMIN    -    Minimum value of array.
C        GMAX    -    Maximum value of array.
C        GINC    -    Contour interval for plotting.
C        NCHAN   -    (unchanged).
C        IERR }  = 0  Successful completion
C             }  =-1  Field has zero range (soft error).
C             }  = 1  End of file encountered.
C             }  = 2  Invalid character encountered in data.
C             }  = 3  End of UTF found.
C             }  = 4  Invalid field or plot type.  
C     Externals:
C        U14UP   -    Unpack the 2D field.
C
C     04.05.93  Mike Blackburn  Argument driven version.
C     05.05.98  Mike Blackburn  Remove IASCII=2 restriction.
C
      IMPLICIT LOGICAL (L)
C
      CHARACTER*50 CHEAD1,CHEAD2,CBLANK,CEND
      INTEGER      IFH(15)
      REAL         DATA(*)
C
      CBLANK(1:50)='                                                  '
      CEND  (1:50)='***END OF DATA***        ***UTF 1.4***            '
      CHEAD1=CBLANK
      CHEAD2=CBLANK
C
C     Read header records.
C
      READ (NCHAN,'(A50)',END=90) CHEAD1(1:50)
      IF (CHEAD1(1:17).EQ.CEND(1:17)) THEN
         IERR=3
         RETURN
      ENDIF
      READ (NCHAN,'(A50)',END=90) CHEAD2(1:50)
      READ (NCHAN,'(15I5)',END=90) (IFH(I),I=1,15)
      READ (NCHAN,'(1P,3E15.7)',END=90) GMIN,GMAX,GINC
C
C     Check plot and field types are valid.
C
      LVALID=.TRUE.
      IF (IFH(3).LT.0) LVALID=.FALSE.
      IF (IFH(3).GT.24) LVALID=.FALSE.
      IF (IFH(3).GE.13.AND.IFH(3).LE.20) LVALID=.FALSE.
      IF (.NOT.LVALID) THEN
         PRINT *,' ***WARNING IN U14RF: INVALID PLOT TYPE = ',IFH(3)
     P          ,' ENCOUNTERED : FIELD NOT READ FROM UTF'
         IERR=4
         RETURN
      ENDIF
C* TEMPORARY
      IF (IFH(3).EQ.0) THEN
         PRINT *,' ***ABORT IN U14RF: PLOT TYPE ZERO REQUIRES AXES'
     P          ,' - NOT YET CODED'
         CALL ABORT
      ENDIF
C
      IF (IFH(5).GT.6) THEN
         PRINT *,' ***WARNING IN U14RF: INVALID FIELD TYPE = ',IFH(5)
     P          ,' ENCOUNTERED : FIELD NOT READ FROM UTF'
         IERR=4
         RETURN
      ENDIF
C* TEMPORARY
      IF (IFH(5).GE.4) THEN
         PRINT *,' ***ABORT IN U14RF: FIELD TYPE .GE.4'
     P          ,' - NOT YET CODED'
         CALL ABORT
      ENDIF
C
C     Read and unpack data array.
C     IERR is set by U14UP.
C
      CALL U14UP(DATA,IFH(1)*IFH(2),IFH(10),GMIN,GMAX,NCHAN,IERR)
      IF (IERR.NE.0) GOTO 99
C
      RETURN
C
C     End of file encountered.
C
   90 CONTINUE
      PRINT *,' ***U14RF IERR=1 : END OF FILE ENCOUNTERED READING'
     P       ,' FIELD ON CHANNEL ',NCHAN
      IERR=1
C
C     Print field header unless blank if there is an error.
C
   99 CONTINUE
      IF (CHEAD1.NE.CBLANK) THEN
         PRINT *,'FIELD IS :'
         PRINT *,CHEAD1(1:50)
         PRINT *,CHEAD2(1:50)
      ENDIF
      RETURN
C
      END
C     ******************************************************************
      SUBROUTINE U14UP(DATA,NPTS,IASCII,GMIN,GMAX,NCHAN,IERR)
C
C     U14UP  : Read and unpack a data array in UTF 1.3 format.
C
C     Each datum is packed as IASCII characters, with the data range
C     split into K**IASCII-1 bins, giving an accuracy of approximately
C     1/(K**IASCII) of the range.
C     Uses the characters 0-9 and lower and upper case A-Z only (K=62).
C     On the first pass, a look-up table is created and saved.
C     *** NOTE : only coded for IASCII=2, 3, or 4.
C     *** NOTE : missing data not yet coded.
C
C     Input arguments:
C        DATA    -    Must be dimensioned NPTS to receive data.
C        NPTS    -    Number of data values.
C        IASCII  -    Number of characters per datum.
C        GMIN    -    Minimum value of array.
C        GMAX    -    Maximum value of array.
C        NCHAN   -    Fortran channel for UTF input.
C     Output arguments:
C        DATA    -    Real data array.
C        IERR }  = 0  Successful completion
C             }  =-1  Field has zero range (soft error).
C             }  = 1  End of file encountered.
C             }  = 2  Invalid character encountered in data.  
C        Other arguments unchanged.
C
C        09.87. M. A. Rowe (FRAM)             Original FRAM packing code
C     03.01.89  Kevin Dunn / Mike Blackburn.  ASCOUT version for UTF 1.3
C     04.05.93  Mike Blackburn                Argument driven version.
C     19.06.95  Mike Blackburn                New look up table based on
C                                             Conrad Winchester's code.
C     05.05.98  Mike Blackburn                Coded for IASCII=2,3,or 4.
C                                             Read in 3-record batches.
C
      PARAMETER(K=62,K2=K*K,K3=K2*K,KLEN=K+1
     *          ,NRLEN=64,NCLEN=3*NRLEN)
      REAL       DATA(NPTS)
      INTEGER    ITAB1(256),IDATA(NCLEN)
      CHARACTER  CDATA*(NCLEN),LKUP*(KLEN)
      LOGICAL    LFIRST
      SAVE ITAB1,LFIRST
      DATA LFIRST/.TRUE./
C
C     ------------------------------------------------------------------
C
      IF (LFIRST) THEN
C
C        On the first call, define a look-up table of characters.
C        Table includes 0-9 and lower and upper case characters A-Z
C        but not control characters (K=62).
C        Characters 1 to K in LKUP are used to represent integer values
C        0 to K-1. The range of the data field is scaled to K**IASCII
C        and IASCII characters used to represent each datum as IASCII
C        digits in base K.
C        (K+1)th character in LKUP is blank for possible future masking.
C
         LKUP( 1:10)='0123456789'
         LKUP(11:36)='ABCDEFGHIJKLMNOPQRSTUVWXYZ'
         LKUP(37:62)='abcdefghijklmnopqrstuvwxyz'
         LKUP(KLEN:KLEN)=' '
C
C        Convert characters to an integer look-up table.
C        The integer value of each character in the collating sequence
C        is used as the index of the table.  Unused characters have a
C        value of 999.
C
         DO 10 I=1,256
            ITAB1(I)=999
   10    CONTINUE
         DO 20 I=1,62
            ICOLL=ICHAR(LKUP(I:I))
            ITAB1(ICOLL)=I
   20    CONTINUE
C
         LFIRST=.FALSE.
C
      ENDIF
C
C     ------------------------------------------------------------------
C
C     Determine number of characters per datum.
C
      IF (IASCII.LT.2) THEN
         PRINT *,' ***ABORT IN U14UP: INVALID IASCII < 2'
         CALL ABORT
      ELSEIF (IASCII.GT.4) THEN
         PRINT *,' ***ABORT IN U14UP: IASCII > 4 NOT CODED'
         CALL ABORT
      ENDIF
C
C     Obtain range and scaling factor.
C
      GAMAX=MAX(ABS(GMAX),ABS(GMIN))
      RANGE=GMAX-GMIN
      ARANG=REAL(K**IASCII-1)
      SCALE=RANGE/ARANG
C
C     Split reading and unpacking into batches of NPBAT points,
C     since packed data array size is defined by this parameter.
C
      NPBAT=NCLEN/IASCII
      NBAT=NPTS/NPBAT
      NLEFT=MOD(NPTS,NPBAT)
      IF (NLEFT.EQ.0) THEN
         NBAT=NBAT-1
         NLEFT=NPBAT
      ELSE
C        Calculate the number of characters and records in any final
C        partial batch.
         NCHAR=NLEFT*IASCII
         NREC=NCHAR/NRLEN
         NREM=MOD(NCHAR,NRLEN)
      ENDIF
C
C     Decode data using subscript of character look-up table
C     to obtain scaled integers.
C     Rescale using range of field to obtain floating point value.
C
      IPTS=0
      NP=NPBAT
      DO 70 IBAT=1,NBAT+1
         IF (IBAT.EQ.NBAT+1) NP=NLEFT
C
         IF (NP.EQ.NPBAT) THEN
            READ(NCHAN,'(A/A/A)',END=90) CDATA(1:NRLEN)
     *         ,CDATA(NRLEN+1:2*NRLEN),CDATA(2*NRLEN+1:3*NRLEN)
         ELSE
            IF (NREC.GT.0) THEN
               DO 30 IREC=1,NREC
                  READ(NCHAN,'(A)',END=90)
     *                CDATA((IREC-1)*NRLEN+1:IREC*NRLEN)
   30          CONTINUE
            ENDIF
            IF (NREM.GT.0) THEN
               READ(NCHAN,'(A)',END=90)
     *             CDATA(NREC*NRLEN+1:NREC*NRLEN+NREM)
            ENDIF
         ENDIF
C
         DO 35 N=1,NP*IASCII
            IDATA(N)=ITAB1(ICHAR(CDATA(N:N)))
   35    CONTINUE
C
         ICMAX=0
         IF (IASCII.EQ.2) THEN
            DO 40 N1=1,NP*2,2
               IPTS=IPTS+1
               I1=IDATA(N1  )
               I0=IDATA(N1+1)
               I=(I1-1)*K+(I0-1)
               DATA(IPTS)=REAL(I)*SCALE+GMIN
               ICMAX=MAX(ICMAX,I1,I0)
   40       CONTINUE
         ELSEIF (IASCII.EQ.3) THEN
            DO 50 N1=1,NP*3,3
               IPTS=IPTS+1
               I2=IDATA(N1  )
               I1=IDATA(N1+1)
               I0=IDATA(N1+2)
               I=(I2-1)*K2+(I1-1)*K+(I0-1)
               DATA(IPTS)=REAL(I)*SCALE+GMIN
               ICMAX=MAX(ICMAX,I2,I1,I0)
   50       CONTINUE
         ELSEIF (IASCII.EQ.4) THEN
            DO 60 N1=1,NP*4,4
               IPTS=IPTS+1
               I3=IDATA(N1  )
               I2=IDATA(N1+1)
               I1=IDATA(N1+2)
               I0=IDATA(N1+3)
               I=(I3-1)*K3+(I2-1)*K2+(I1-1)*K+(I0-1)
               DATA(IPTS)=REAL(I)*SCALE+GMIN
               ICMAX=MAX(ICMAX,I3,I2,I1,I0)
   60       CONTINUE
         ENDIF
C
         IF (ICMAX.EQ.999) GOTO 91
   70 CONTINUE
C
C     Check that correct number of data points have been processed.
C
      IF (IPTS.NE.NPTS) THEN
         PRINT *,'***ABORT IN U14UP: WRONG COUNTER SUM: IPTS,NPTS='
     *          ,IPTS,NPTS
         CALL ABORT
      ENDIF
C
C     Print warning if range of field is zero.
C     This is treated as a soft error (IERR<0).
C     Otherwise return successfully.
C
      IERR=0
      IF (RANGE.LE.GAMAX*1.E-10) THEN
         PRINT *,' ***U14UP WARNING : FIELD HAS ZERO RANGE.'
         IERR=-1
      ENDIF
      RETURN
C
C     End of file encountered.
C
   90 CONTINUE
      PRINT *,' ***U14UP IERR=1 : END OF FILE ENCOUNTERED READING'
     P       ,' FIELD ON CHANNEL ',NCHAN
      IERR=1
      RETURN
C
C     Invalid character encountered in data.
C
   91 CONTINUE
      PRINT *,' ***U14UP IERR=2 : INVALID CHARACTER ENCOUNTERED READING'
     P       ,' DATA ON CHANNEL ',NCHAN
      PRINT *,' RECORD IS :'
      WRITE(*,'(A)') CDATA(1:2*NP)
      IERR=2
      RETURN
C
      END
C     ******************************************************************
      SUBROUTINE UTFWI(I1,I2,I3,I4,I5,I6,I7,I8,I9,I10
     S                ,I11,I12,I13,I14,I15,IH)
C
C     UTFWI  : "Write" 15 integers into an integer array for a UTF
C              version 1.3 or 1.4 global or field header.
C
C     Saves 15 lines of code in calling programs.
C     *** Note : there is currently no checking of the validity of the
C                input integers.
C
C     Input arguments:
C        I1-I15  -    Individual integers.
C        IH      -    Unset integer array.
C     Output arguments
C        I1-I15  -    Unchanged.
C        IH      -    Integer array containing I1 to I15 in order.
C
C     18.07.96  Mike Blackburn  Argument driven version.
C
      INTEGER IH(15)
C
      IH(1)=I1
      IH(2)=I2
      IH(3)=I3
      IH(4)=I4
      IH(5)=I5
      IH(6)=I6
      IH(7)=I7
      IH(8)=I8
      IH(9)=I9
      IH(10)=I10
      IH(11)=I11
      IH(12)=I12
      IH(13)=I13
      IH(14)=I14
      IH(15)=I15
C
      RETURN
      END
C     ******************************************************************
      SUBROUTINE UTFRI(I1,I2,I3,I4,I5,I6,I7,I8,I9,I10
     S                ,I11,I12,I13,I14,I15,IH)
C
C     UTFRI  : "Read" 15 integers from an integer array for a UTF
C              version 1.3 or 1.4 global or field header.
C
C     Saves 15 lines of code in calling programs.
C     *** Note : there is currently no checking of the validity of the
C                output integers.
C
C     Input arguments:
C        I1-I15  -    Individual integers, unset.
C        IH      -    Integer array containing UTF header integers.
C     Output arguments
C        I1-I15  -    Unpacked individual integers.
C        IH      -    Unchanged.
C
C     18.07.96  Mike Blackburn  Argument driven version.
C
      INTEGER IH(15)
C
      I1=IH(1)
      I2=IH(2)
      I3=IH(3)
      I4=IH(4)
      I5=IH(5)
      I6=IH(6)
      I7=IH(7)
      I8=IH(8)
      I9=IH(9)
      I10=IH(10)
      I11=IH(11)
      I12=IH(12)
      I13=IH(13)
      I14=IH(14)
      I15=IH(15)
C
      RETURN
      END
C     ******************************************************************
      SUBROUTINE U13GV(FIN,FOUT,ALAT,MG,JG,NHEM)
C
C     U13GV  : Convert a (MG,JG*NHEM) lat-long field to UTF 1.3 format
C              by repeating first longitude and adding equatorial row.
C
C     Input arguments:
C        FIN     -    Input field, dimension (MG,JG*NHEM).
C        ALAT    -    Latitudes in degrees.
C        MG      -    Number of longitudes.
C        JG      -    Number of latitudes from pole to equator.
C        NHEM    -    Number of hemispheres.
C     Output arguments:
C        FOUT    -    Output field, dimension (MG+1,JG*NHEM+1).
C     Other arguments unchanged.
C
C     02.05.93  Mike Blackburn  Argument driven version.
C
      REAL FIN(MG,JG*NHEM),FOUT(MG+1,JG*NHEM+1),ALAT(JG)
C
      MGP=MG+1
      JGP=JG+1
      JGG=JG*NHEM
      JGGP=JGG+1
C
C     Constants for hemispheric case to extrapolate quadratically to
C     equator with zero gradient there.
C
      EXE1=ALAT(JG)*ALAT(JG)
      EXE2=ALAT(JG-1)*ALAT(JG-1)
      EXE3=1./(EXE2-EXE1)
C
C     Copy (MG,JG*NHEM) part of field, shifting southern hemisphere.
C
      DO 20 J=1,JG
         DO 10 I=1,MG
            FOUT(I,J)=FIN(I,J)
            IF (NHEM.EQ.2) FOUT(I,JGGP+1-J)=FIN(I,JGGP-J)
   10    CONTINUE
   20 CONTINUE
C
C     If hemispheric, interpolate to equator.  If global, extrapolate.
C
      IF (NHEM.EQ.1) THEN
         DO 30 I=1,MG
            FOUT(I,JGP)=EXE3*(EXE2*FOUT(I,JG)-EXE1*FOUT(I,JG-1))
   30    CONTINUE
      ELSE
         DO 40 I=1,MG
            FOUT(I,JGP)=0.5*(FOUT(I,JG)+FOUT(I,JG+2))
   40    CONTINUE
      ENDIF
C
C     Copy first longitude into (MG+1) position.
C
      DO 50 J=1,JGGP
         FOUT(MGP,J)=FOUT(1,J)
   50 CONTINUE
C
      RETURN
      END
C     ******************************************************************
      SUBROUTINE U13GIN(FIN,FOUT,ALAT,MG,JG,NHEM)
C
C     U13GIN : Convert a (MGP,JG*NHEM+1) lat-long field in UTF 1.3
C              format to (MG,JG*NHEM) by removing entire equatorial row
C              and copy of first longitude.
C
C     Input arguments:
C        FIN     -    Input field, dimension (MGP,JG*NHEM+1).
C        ALAT    -    Latitudes in degrees.
C        MG      -    Number of longitudes.
C        JG      -    Number of latitudes from pole to equator.
C        NHEM    -    Number of hemispheres.
C     Output arguments:
C        FOUT    -    Truncated field, dimension (MG,JG*NHEM).
C     Other arguments unchanged.
C
C     15.10.93  Mike Blackburn  Argument driven version.
C
      REAL FIN(MG+1,JG*NHEM+1),FOUT(MG,JG*NHEM),ALAT(JG)
C
      MGP=MG+1
      JGP=JG+1
      JGG=JG*NHEM
      JGGP=JGG+1
C
C     Copy (MG,JG*NHEM) part of field, shifting southern hemisphere.
C
      DO 20 J=1,JG
         DO 10 I=1,MG
            FOUT(I,J)=FIN(I,J)
            IF (NHEM.EQ.2) FOUT(I,JGGP-J)=FIN(I,JGGP+1-J)
   10    CONTINUE
   20 CONTINUE
C
      RETURN
      END
C     ******************************************************************
C
C23456789 123456789 123456789 123456789 123456789 123456789 123456789 12
C
